import datetime
import re


def date_to_str(str_date: str, portal: str) -> str:
    """
    날짜 형식 DataBase Format 처리 작업
    Args:
        str_date (str): date formated string text by portal site
    Returns:
        str: processed text
    """
    if portal == "daum":
        if str_date == "" or str_date == " " or str_date is None:
            str_date = str(datetime.datetime.now())
            str_date = str_date[:19]
            str_date = datetime.datetime.strptime(str_date, "%Y-%m-%d %H:%M:%S")
        else:
            str_date = datetime.datetime.strptime(
                str_date, "%Y. %m. %d. %H:%M"
            ).strftime("%Y-%m-%d %H:%M:%S")
    elif portal == "naver":
        if str_date == "" or str_date == " " or str_date is None:
            str_date = str(datetime.datetime.now())
            str_date = str_date[:19]
            str_date = datetime.datetime.strptime(str_date, "%Y-%m-%d %H:%M:%S")

        elif str_date.endswith("일전"):
            datenum = str_date.replace("일전", "").replace(" ", "")
            str_date = str(
                datetime.datetime.now() - datetime.timedelta(days=int(datenum))
            )
            str_date = str(str_date[:11]) + "00:00:00"

        elif str_date.endswith("시간전"):
            datenum = str_date.replace("시간전", "").replace(" ", "")
            str_date = str(
                datetime.datetime.now() - datetime.timedelta(hours=int(datenum))
            )
            str_date = str(str_date[:19])

        elif str_date.endswith("분전"):
            datenum = str_date.replace("분전", "").replace(" ", "")
            str_date = str(
                datetime.datetime.now() - datetime.timedelta(minutes=int(datenum))
            )
            str_date = str(str_date[:19])

        else:
            if str_date.startswith("기사입력"):
                str_date = str_date.replace("기사입력 ", "")
            str_date = (
                str_date.replace("오전", "AM")
                .replace("오후", "PM")
                .replace("오 전", "AM")
                .replace("오 후", "PM")
            )
            str_date = datetime.datetime.strptime(
                str_date, "%Y.%m.%d. %p %I:%M"
            ).strftime("%Y-%m-%d %H:%M:%S")
    return str(str_date)


def emoji_processing(content: str) -> str:
    """
    이모지 처리
    Args:
        content (str): string content

    Returns:
        str: processed content
    """

    emoji_pattern = re.compile(
        "["
        u"\U0001F600-\U0001F64F"  # emoticons
        u"\U0001F300-\U0001F5FF"  # symbols & pictographs
        u"\U0001F680-\U0001F6FF"  # transport & map symbols
        u"\U0001F1E0-\U0001F1FF"  # flags (iOS)
        "]+",
        flags=re.UNICODE,
    )
    content = emoji_pattern.sub(r"", str(content))  # no emoji

    return content


def news_preprocessing(content: str) -> str:
    """
    특수문자 및 본문 외 불필요 텍스트 제거 작업
    Args:
        content (str): string content

    Returns:
        str: processed content
    """

    result = (
        str(content)
        .replace("뉴스코리아", "")
        .replace("및", "")
        .replace("Copyright", "")
        .replace("copyright", "")
        .replace("COPYRIGHT", "")
        .replace("저작권자", "")
        .replace("ZDNET A RED VENTURES COMPANY", "")
        .replace("appeared first on 벤처스퀘어", "")
        .replace("appeared first on 벤처 스퀘어", "")
        .replace("appeared first on 모비인사이드 MOBIINSIDE", "")
        .replace("appeared first on 모비 인사이드 MOBIINSIDE", "")
        .replace("The post", "")
        .replace(".", " ")
    )
    result = re.sub(
        "http[s]?://(?:[a-zA-Z]|[0-9]|[$\-@\.&+:/?=]|[!*\(\),]|(?:%[0-9a-fA-F][0-9a-fA-F]))+",
        "",
        result,
    )
    result = re.sub(r"[a-zA-Z가-힣]+뉴스", "", result)
    result = re.sub(r"[a-zA-Z가-힣]+ 뉴스", "", result)
    result = re.sub(r"[a-zA-Z가-힣]+newskr", "", result)
    result = re.sub(r"[a-zA-Z가-힣]+Copyrights", "", result)
    result = re.sub(r"[a-zA-Z가-힣]+ Copyrights", "", result)
    result = re.sub(r"\s+Copyrights", "", result)
    result = re.sub(r"[a-zA-Z가-힣]+com", "", result)
    result = re.sub(r"[가-힣]+ 기자", "", result)
    result = re.sub(r"[가-힣]+기자", "", result)
    result = re.sub(r"[가-힣]+ 신문", "", result)
    result = re.sub(r"[가-힣]+신문", "", result)
    result = re.sub(r"데일리+[가-힣]", "", result)
    result = re.sub(r"[가-힣]+투데이", "", result)
    result = re.sub(r"[가-힣]+미디어", "", result)
    result = re.sub(r"[가-힣]+ 데일리", "", result)
    result = re.sub(r"[가-힣]+데일리", "", result)
    result = re.sub(r"[가-힣]+ 콘텐츠 무단", "", result)
    result = re.sub(r"전재\s+변형", "전재", result)
    result = re.sub(r"[가-힣]+ 전재", "", result)
    result = re.sub(r"[가-힣]+전재", "", result)
    result = re.sub(r"[가-힣]+배포금지", "", result)
    result = re.sub(r"[가-힣]+배포 금지", "", result)
    result = re.sub(r"\s+배포금지", "", result)
    result = re.sub(r"\s+배포 금지", "", result)
    result = re.sub(r"[a-zA-Z가-힣]+.kr", "", result)
    result = re.sub(r"/^[a-z0-9_+.-]+@([a-z0-9-]+\.)+[a-z0-9]{2,4}$/", "", result)
    result = re.sub(r"[\r|\n]", "", result)
    result = re.sub(r"\[[^)]*\]", "", result)
    result = re.sub(r"\([^)]*\)", "", result)
    result = re.sub(r"[^ ㄱ-ㅣ가-힣A-Za-z0-9]", "", result)
    result = re.sub(r"이 글은 외부 필자인 +[a-zA-Z가-힣]", "", result)
    result = re.sub(r"[a-zA-Z가-힣]+기고입니다.", "", result)
    if result.find("관련 기사") != -1 or result.find("관련기사") != -1:
        result = result[: result.find("관련기사")]
        result = result[: result.find("관련 기사")]
    result = result.strip()

    return result



import re

from webdriver_manager.chrome import ChromeDriverManager

from selenium import webdriver
from selenium.webdriver.remote.webdriver import WebDriver


class Helper:
    def __init__(self, url: str, timeout=60, page_timeout=5, headless=True):
        self.timeout = timeout
        self.driver: WebDriver = None
        chrome_options = webdriver.ChromeOptions()

        if headless:
            chrome_options.add_argument("--headless")
        chrome_options.add_argument("--no-sandbox")
        chrome_options.add_argument("--disable-dev-shm-usage")
        chrome_options.add_argument("--disable-gpu-usage")

        # install driver
        self.driver = webdriver.Chrome(
            ChromeDriverManager().install(), options=chrome_options
        )

        self.driver.set_page_load_timeout(page_timeout)
        try:
            self.driver.get(url)
        except Exception as e:
            print(e)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if self.driver:
            self.driver.quit()
            self.driver = None

    def get_text_by_xpath(self, xpath: str) -> str:
        """
        Args:
            xpath (str): html xpath
        Returns:
            str: html tag text
        """

        tag_text = None
        try:
            self.driver.implicitly_wait(self.timeout)
            tag_text = self.driver.find_element_by_xpath(xpath).text
        except Exception as e:
            print(e)
        return tag_text

    def click_by_xpath(self, xpath: str):
        """
        Args:
            xpath(str): html xpath
        Returns:
            void
        """
        try:
            self.driver.implicitly_wait(self.timeout)
            self.driver.find_element_by_xpath(xpath).click()
        except Exception as e:
            print(e)

    def get_texts_by_xpath(self, xpath: str) -> list:
        """
        Args:
            xpath(str): html xpath
        Returns:
            list: element text list
        """
        p = re.compile(".+ol$|.+ul$")
        m = p.match(xpath)
        text_list = []
        self.driver.implicitly_wait(self.timeout)
        try:
            if m:
                for element in self.driver.find_elements_by_xpath(xpath + "/li"):
                    text_list.append(element.text)
            else:
                for element in self.driver.find_elements_by_xpath(xpath):
                    text_list.append(element.text)
        except Exception as e:
            print(e)
        return text_list

    def go_to(self, url: str):
        """
        Args:
            url(str): Page URL
        Returns:
            void: None
        """
        try:
            self.driver.execute_script("location.href='{}'".format(url))
        except Exception as e:
            print(e)

    def back(self):
        """
        Returns:
            void: None
        """
        try:
            self.driver.back()
        except Exception as e:
            print(e)

    def get_attribute_by_xpath(self, xpath: str, attribute_name: str) -> str:
        """
        Args:
            xpath(str): html xpath
            attribute_name(str): html 속성명
        Returns:
            str: html tag text
        """
        tag_attribute = None
        try:
            self.driver.implicitly_wait(self.timeout)
            tag_attribute = self.driver.find_element_by_xpath(xpath).get_attribute(
                attribute_name
            )
        except Exception as e:
            print(e)
        return tag_attribute



from urllib.parse import quote


def make_data_source(db_info, db_name):
    """
    Args:
        database_uri 생성 및 sqlalchemy engine 생성
        Args:
            db_info(str): 사용할 데이터베이스 서버 정보
            db_name(str): 사용할 데이터베이스 이름
    Returns:
        str: data_source
    """
    db_id = db_info["id"]
    password = db_info["pwd"]
    host = db_info["ip"]
    port = db_info["port"]

    data_source = (
        "mysql+pymysql://"
        + db_id
        + ":"
        + quote(password)
        + "@"
        + host
        + ":"
        + str(port)
        + "/"
        + db_name
    )
    return data_source



import pandas as pd
import pymysql
import sqlalchemy
from pshmodule.db.config import make_data_source


class DataSource:
    def __init__(self, db_info: dict, db_name: str):
        """
        database_uri 생성 및 sqlalchemy engine 생성
        Args:
            db_info(str): 사용할 데이터베이스 서버 정보
            db_name(str): 사용할 데이터베이스 이름
        """
        self.table_dict = {}
        # sqlalchemy
        self.database_uri = make_data_source(db_info, db_name)
        self.engine = sqlalchemy.create_engine(self.database_uri, pool_pre_ping=True)

        # pymysql
        self.conn = pymysql.connect(
            user=db_info["id"],
            passwd=db_info["pwd"],
            db=db_name,
            host=db_info["ip"],
            port=int(db_info["port"]),
            charset="utf8",
            use_unicode=True,
        )

    def __enter__(self):
        return self.engine.begin()

    def __exit__(self, exc_type, exc_val, exc_tb):
        if self.engine:
            self.engine.dispose()
        if self.conn:
            self.conn.close()
        print("-------------------------------" * 6)

    def __del__(self):
        if self.engine:
            self.engine.dispose()
        if self.conn:
            self.conn.close()
        print("-------------------------------" * 6)

    def df_to_sql(self, df: pd.DataFrame, table_name: str):
        """
        dataframe 형식으로 database insert
        Args:
            df(DataFrame): 데이터베이스에 저장할 데이터프레임 객체
            table_name(str): 테이블 이름
        """
        print("execute start")
        df.to_sql(table_name, con=self.engine, if_exists="append", index=False)
        print("execute end")

    def execute_query(self, query: str):
        """
        simply query excute
        Args:
            query(str): 실행할 쿼리
        """
        try:
            print("execute start")
            with self.engine.begin() as con:
                result = con.execute(query)
                print("execute end")
                return result
        except Exception as e:
            print(e)

    def executemany_query(self, query: str, param: list):
        """
        pymysql excutemany 대량 query 실행
        Args:
            query(str): 실행할 쿼리
            param(list): 쿼리 set, where id 파라미터

            ex) query : "update table set column=%s where id=%s;"
                param : [('김', '1'), ('이', '2')]
        """
        try:
            print("executemany start")
            with self.conn.cursor(pymysql.cursors.DictCursor) as cursor:
                result = cursor.executemany(query, param)
                self.conn.commit()
                print("executemany end")
                return result
        except Exception as e:
            print(e)

    def select_query_to_df(self, query: str):
        """
        select 쿼리 결과를 데이터프레임 형태로 반환
        Args:
            query(str): 실행할 쿼리
        """
        try:
            print("excute start")
            df = pd.read_sql(
                query,
                con=self.engine,
            )

            print(f"df's length : {len(df)}")
            print("excute end")

        except Exception as e:
            print(e)
        return df



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pshmodule",  # 프로젝트 이름
    version="0.1.11",  # 프로젝트 버전
    description="Modules related to preprocessing, crawling, collection, database, data load & save ",  # 간단한 설명
    url="https://github.com/hipster4020/package-module",  # 프로젝트 주소
    author="shpark610",  # 작성자
    author_email="hipster4020@gmail.com",  # 작성자 이메일
    long_description=long_description,  # 프로젝트 설명, 보통 README.md로 관리
    long_description_content_type="text/markdown",
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    python_requires=">=3.8",
    install_requires=[  # 설치시 설치할 라이브러리
        "sqlalchemy",
        "webdriver_manager",
        "selenium",
        "openpyxl",
    ],
)



import os
import pickle

import openpyxl
import json
import pandas as pd


def load(path: str = ""):
    """
    확장자 별 file load
    Args:
        path (str): file path
    Returns:
        str: load data
    """
    check = os.path.isfile(path)
    try:
        data = []
        if check:
            extension = os.path.splitext(path)[1]
            print(f"extension : {extension}")

            # text
            if extension in (".txt", ".csv", ".tsv"):
                data = pd.read_csv(path, encoding="utf-8")
            # excel
            elif extension in (".xlsx", ".xls"):
                data = pd.read_excel(path)
            # pickle
            elif extension == ".pickle":
                with open(path, "rb") as f:
                    data = pickle.load(f)
            # json
            elif extension == ".json":
                """
                Read list of objects from a JSON lines file.
                """
                with open(path, 'r', encoding='utf-8') as f:
                    for line in f:
                        data.append(json.loads(line.rstrip('\n|\r')))
                data = pd.DataFrame(data)
            print("Loaded {} records from {}".format(len(data), path))
        else:
            raise print("The file does not exist.")
        return data
    except Exception as e:
        print(e)


def save(path: str, data):
    """
    확장자 별 file save
    Args:
        path (str): file path
        data : input data
    """
    check = os.path.isfile(path)
    try:
        if not check:
            extension = os.path.splitext(path)[1]

            # text
            if extension in (".txt"):
                with open(path, "w") as f:
                    for i in data:
                        f.write(str(i) + "\n")
            # excel
            elif extension in (".xlsx", ".xls"):
                data.to_excel(path)
            # pickle
            elif extension == ".pickle":
                with open(path, "wb") as f:
                    pickle.dump(data, f, pickle.HIGHEST_PROTOCOL)
            # json
            elif extension == ".json":
                """
                Write list of objects to a JSON lines file for DeepLearning.
                """
                if 'label' in data.columns or 'content' in data.columns:
                    temp_dict = [{"content": row['content'].strip(), "label": row['label']} for _, row in data.iterrows()]
                    with open(path, 'w', encoding='utf-8') as f:
                        for line in temp_dict:
                            json_record = json.dumps(line, ensure_ascii=False)
                            f.write(json_record + '\n')
                else:
                    raise print("DataFrame' columns name no have 'content' or 'label'")
            print("Saved {} records".format(len(data)))
        else:
            raise print("file exists")
    except Exception as e:
        print(e)
